"""
zxx.File
文件类
    针对外部的视频或音频文件进行基本操作。
"""


from __future__ import annotations
from moviepy.editor import *
from os.path import join

from .options import GetPath
from .tools import str2sec


class File:
    def __init__(self, filename: str, folder: str = None) -> None:
        """
        zxx.File 文件类，针对外部的视频或音频文件进行基本操作。

        参数说明：
            filename：新建 zxx.File 类的一个实例时，必须指定文件名。文件名是相对于 folder 的相对路径。
            folder：视频文件所在的文件夹绝对路径。如果未指定，则默认为工作目录（即 zxx.options.GetPath() 的返回值）。
        """
        self.__filename = filename
        self.__folder = folder
        if folder == None:
            self.__folder = GetPath()
        self.__path = join(self.__folder, self.__filename)
        self.__contents = VideoFileClip(self.__path)

    def contents(self) -> VideoFileClip:
        """
        获取文件内容。返回值类型是 moviepy.video.io 的 VideoFileClip 类。
        """
        return self.__contents

    def select(self, begin: str, finish: str) -> VideoClip:
        """
        截取视频文件中的一段，并根据这段内容生成 moviepy.video 的 VideoClip 类的一个实例。

        参数说明：
            begin：开始时间点，可以写成 "1:03:13" 这样的形式。
            finish：结束时间点，写法同上。注意开始时刻必须早于结束时刻。
        """
        b = str2sec(begin)
        e = str2sec(finish)
        if b >= e:
            raise Exception("时间轴错误：开始时刻 %s 未能早于结束时刻 %s" % (begin, finish))
        else:
            clip = self.contents().subclip(b, e)
        return clip



"""
zxx.tools
小工具
    供其他模块使用的一些工具性函数，不建议在剪辑集锦时直接调用。
"""


from moviepy.editor import *
import cv2
from PIL import ImageFont, ImageDraw, Image
import numpy as np
from os import path

from .options import GetPath, GetMatchInfo, GetScoreBoardStyle


def str2sec(string: str) -> float:
    """
    字符串转秒数。

    例如：
        str2sec("1:35:36.3")
            返回值：5736.3
    """
    time = string.strip().split(":")
    if len(time) == 3:
        h, m, s = time
    elif len(time) == 2:
        h = 0
        m, s = time
    elif len(time) == 1:
        h, m = 0, 0
        s = time[0]
    else:
        return None
    seconds = 3600 * float(h) + 60 * float(m) + float(s)

    return seconds


def sec2str(seconds: float) -> str:
    """
    秒数转字符串，秒数保留最多三位小数，使用去尾法而非四舍五入。

    例如：
        sec2str(367.57898)
            返回值："06:07.578"
        sec2str(367)
            返回值："06:07"
    """
    m, s = divmod(seconds, 60)

    # 秒数前面补0
    parts = str(s).split(".")
    if len(parts[0]) == 1:
        parts[0] = "0" + parts[0]
    
    # 处理含小数的秒数
    if "." in str(s):
        if len(parts[1]) > 3:
            parts[1] = parts[1][:3]
        s = parts[0] + "." + parts[1]
    else:
        s = parts[0]
    if m == 0:
        return s
    
    h, m = divmod(m, 60)
    if h == 0:
        string = "%02d:%s" % (m, s)
    else:
        string = "%02d:%s" % (h, m, s)
    
    return string


def add_txt_to_img_center(filename: str, text: str, fontpath: str, color: str | tuple):
    """
    在图片正中央添加文字。

    参数说明：
        filename：要加字的图片文件名，工作路径（即 zxx.options.GetPath() 返回值）的相对路径。
        text：要加的文字。
        fontpath：字体文件的路径，注意是绝对路径。
        color：字体颜色，参数值会直接传送给 PIL.ImageDraw.text() 方法的 fill 参数，
            使用色彩名、十六进制、RGB、RGBA 等均可，可采用以下写法中的任意一种：
                color="pink"
                color="#800080"
                color=(255, 10, 10)
                color=(255, 10, 10, 100)
    """

    # 导入图片（解决中文路径乱码问题）
    img_path = path.join(GetPath(), filename)
    img = cv2.imdecode(np.fromfile(img_path, dtype=np.uint8), cv2.IMREAD_UNCHANGED)

    # 读取图片，并手动将 BGR(A) 调整为 RGB(A)
    if filename[-3:] == "png":
        b_channel, g_channel, r_channel, a_channel = cv2.split(img)
        img_RGBA = cv2.merge((r_channel, g_channel, b_channel, a_channel))
        img_pil = Image.fromarray(img_RGBA)
    else:
        b_channel, g_channel, r_channel = cv2.split(img)
        img_RGB = cv2.merge((r_channel, g_channel, b_channel))
        img_pil = Image.fromarray(img_RGB)
    
    # 确定合适的字号
    width, height = img_pil.size
    fontsize = min(int(0.8 * width / len(text)), int(0.8 * height))
    font = ImageFont.truetype(fontpath, fontsize)

    # 精确处理文字位置（居中）
    x_0, y_0, x_1, y_1 = font.getbbox(text)    # 获取边界框，处理可能的 offset
    position = ((width - x_1 - x_0) / 2, (height - y_1 - y_0) / 2)
    draw = ImageDraw.Draw(img_pil)
    draw.text(position, text, font=font, fill=color)
    img = np.array(img_pil)
    
    return img


def add_effects(clip: VideoClip, speed: float = 1, silence: bool = False, lum: float = 0, 
            contrast: float = 0, fadein: float = 0, fadeout: float = 0) -> VideoClip:
    """
    为视频片段添加视频特效。

    参数说明：
        clip：要处理的视频片段。类型是 moviepy.video 的 VideoClip 类。
        speed：调整视频的速度倍数，不建议放慢太多，可能导致时间轴错误。
        silence：是否消音。
        lum：亮度要增加或减少的值，大小没有限制，不过一般在 -127 至 127 之间，可自行调试。
        contrast：对比度要调整的值，大小没有限制，不过一般在 -1 至 1之间，可自行调试。
        fadein：淡入效果持续的秒数。
        fadeout：淡出效果持续的秒数。
    """

    # 亮度和对比度
    if lum != 0 or contrast != 0:
        clip = vfx.lum_contrast(clip, lum=lum, contrast=contrast)
    # 变速
    if speed != 1:
        clip_dur = clip.duration
        clip = clip.fl_time(lambda t:speed*t)
        clip = clip.set_duration(clip_dur / speed)
    # 淡入和淡出（变速之后）
    if fadein != 0:
        clip = clip.fx(vfx.fadein, fadein)
    if fadeout != 0:
        clip = clip.fx(vfx.fadeout, fadeout)
    # 消音
    if silence:
        clip = clip.without_audio()

    return clip


def add_caption(clip: VideoClip, text: str) -> VideoClip:
        """
        给视频片段加字幕。
        字幕的字体、字号、颜色、位置等参数，通过 zxx.options.SetCaptionStyle() 统一设置。

        参数说明：
            clip：要加字幕的视频片段。类型是 moviepy.video 的 VideoClip 类。
            text：字幕文案。可以为空字符串。
        """
        from .options import GetCaptionStyle

        # 如果 text 为空，会报错，因此需要打一个空格
        if text == "":
            text = " "
        
        # 按照宽度为 1920 像素的视频的字号，调整字号大小
        video_width = clip.size[0]
        fontsize = GetCaptionStyle("fontsize") / 1920 * video_width
        text_clip = TextClip(
            text,
            font     = GetCaptionStyle("font"),
            fontsize = fontsize,
            color    = GetCaptionStyle("color"),
        ).set_position(GetCaptionStyle("position"), relative=GetCaptionStyle("relative"))
        clip_dur = clip.duration
        clip = CompositeVideoClip([clip, text_clip]).set_duration(clip_dur)
        
        return clip


def add_scoreboard(clip: VideoClip, home: int = 0, away: int = 0) -> VideoClip:
    """
    加比分牌。显示的队名通过 zxx.options.SetMatchInfo() 设置。

    参数说明：
        clip：要加比分牌的视频片段。类型是 moviepy.video 的 VideoClip 类。
        home：主队当前得分。
        away：客队当前得分。
    """

    # 生成比分牌文字内容
    home_name = GetMatchInfo("home")
    away_name = GetMatchInfo("away")
    text = f"{home_name}　{home}-{away}　{away_name}"
    fontpath = path.join(GetPath(), GetScoreBoardStyle("font_file"))
    # 合成比分牌图片
    img = add_txt_to_img_center(
        filename = GetScoreBoardStyle("image"),
        text = text,
        fontpath = fontpath,
        color = GetScoreBoardStyle("color") 
    )
    # 合成图片和视频，比分牌图片的宽度是视频的 1/4
    clip_width = clip.size[0]
    score_width = 0.25 * clip_width
    score_clip = ImageClip(img).fx(vfx.resize, width=score_width)
    # 调整比分牌显示时长
    clip_dur = clip.duration
    clip = CompositeVideoClip([clip, score_clip]).set_duration(clip_dur)

    return clip



"""
Python package zxx 1.1.0
Latest update: 2025-07-30

各模块调用顺序：
    Highlight <- File <- tools <- options

后续有待解决的事项：
    1. 可以增加一个标准化音量的功能。
    2. SetCaptionStyle() 和 SetScoreBoardStyle() 指定字体属性的方式不一样，一个是基于 ImageMagick，一个是基于 Pillow，使用起来不统一。
"""


from .File import File
from .Highlight import Highlight



"""
zxx.Highlight
集锦类
    由视频片段处理组合形成的视频集锦。
"""


from __future__ import annotations
from moviepy.editor import *
from os import path

from .options import GetPath, SetMatchInfo
from .tools import str2sec, sec2str, add_effects, add_caption, add_scoreboard
from .File import File


class Highlight():
    def __init__(self, contents: VideoClip = None):
        """
        zxx.Highlight 集锦类，由多个经过处理的视频片段组合形成集锦，并对集锦进行配置背景音乐、刷新比分等进一步操作。

        参数说明：
            contents：集锦的初始内容，类型是 moviepy.video 的 VideoClip 类。新建集锦实例时，建议不指定 contents。
        """
        self.__contents = contents
        self.__source_file = None
        self.__score = (0, 0)
        self.__show_score = False

    # 以下方法和 zxx.Vedio 类是一样的

    def contents(self) -> VideoClip | None:
        """
        获取视频内容。
        """
        return self.__contents

    def duration(self) -> str:
        """
        获取视频时长。
        """
        dur = sec2str(self.contents().duration)
        return dur

    def print_duration(self) -> Highlight:
        """
        打印视频时长。
        返回值是 self，即调用该函数的实例本身，因此可用于链式写法。
        """
        print(self.duration())
        return self

    # 以下是 Highlight 类独有的方法

    def source_file(self) -> File:
        """
        返回集锦当前正在处理的视频片段的源文件，类型为 zxx.File。
        """
        return self.__source_file

    def score(self) -> tuple:
        """
        返回当前比分（主队得分在前）。
        """
        return self.__score

    def __add_video(self, clip: VideoClip) -> Highlight:
        """
        【不建议外部调用本方法，实际剪辑集锦时，应使用 zxx.Highlight.use() 代替】

        向集锦中追加一段视频片段。如果新追加视频画面尺寸不同，会自动将新视频统一成和已有集锦相同的尺寸。
        返回值是 self，即调用该函数的实例本身，因此可用于链式写法。

        参数说明：
            video：要追加的视频片段，类型是 moviepy.video 的 VideoClip 类。
        """
        if self.contents() == None:
            self.__contents = clip
        else:
            # 统一视频尺寸
            hl_size = self.__contents.size
            v_size = clip.size
            if hl_size != v_size:
                new_clip = clip.resize(hl_size)
            else:
                new_clip = clip
            self.__contents = concatenate_videoclips([self.contents(), new_clip])
        return self

    def silence(self) -> Highlight:
        """
        集锦整体消音。
        返回值是 self，即调用该函数的实例本身，因此可用于链式写法。
        """
        clip = self.contents()
        self.__contents = clip.without_audio()
        return self

    def add_bgm(self, filename: str, folder: str = None, 
                select: list = [], repeat: int = 1, mode = "cut") -> Highlight:
        """
        为集锦添加背景音乐。
        返回值是 self，即调用该函数的实例本身，因此可用于链式写法。

        参数说明：
            filename：背景音乐的文件名。
            folder：背景音乐文件所在的文件夹绝对路径。如果未指定，则默认为工作目录（即 zxx.options.GetPath() 的返回值）。
            select：选择截取音乐的时间范围。
                写成 ["02:12", "03:06"] 这样的形式。注意开始时刻必须早于结束时刻。
                开始 / 结束时刻如果是空字符串，默认是音乐的开始 / 结束处。
            repeat：设置把截取的音乐片段重复几次（默认为 1，即不重复）
            mode：设置配乐的模式。具体有如下选择：
                mode = "cut"：原速播放音乐，视频结束就停止音乐。
                mode = "change_music_speed"：自动调整音乐速度，以匹配视频时长。可能导致音乐变调。
        """
        if folder == None:
            folder = GetPath()
        video_clip = self.__contents
        audio_clip = AudioFileClip(path.join(folder, filename))
        if select != []:
            if select[0] == "":
                b = 0
            else:
                b = str2sec(select[0])
            if select[1] == "":
                e = audio_clip.duration
            else:
                e = str2sec(select[1])
            if b >= e:
                raise Exception("背景音乐剪辑错误：开始时间（%s）不早于结束时间（%s）" % (b, e))
            else:
                audio_clip = audio_clip.subclip(b, e)
        if repeat > 1:
            for i in range(repeat - 1):
                audio_clip = concatenate_audioclips([audio_clip, audio_clip])
        video_duration = video_clip.duration
        audio_duration = audio_clip.duration
        if mode == "cut":               # 视频结束停止音乐
            audio_clip = audio_clip.subclip(0, video_duration)
            self.__contents = video_clip.set_audio(audio_clip)
        elif mode == "change_music_speed":    # 调整音乐速度，匹配视频时长
            speed = audio_duration / video_duration
            audio_clip = audio_clip.fl_time(lambda t:speed*t)
            audio_clip = audio_clip.set_duration(audio_duration / speed)
            self.__contents = video_clip.set_audio(audio_clip)
            print("为匹配视频长度，背景音乐速度已调整为原先的 %.2f 倍" % speed)
        else:
            raise Exception("添加背景音乐失败，请指定模式（\"cut\" 或 \"change_music_speed\"）")
        return self

    def use(self, filename: str, folder: str = None) -> Highlight:
        """
        指定集锦的下一段视频片段使用的视频源文件。
        返回值是 self，即调用该函数的实例本身，因此可用于链式写法。
        
        参数说明：
            filename：视频文件名。
            folder：视频源文件所在的文件夹绝对路径。如果未指定，则默认为工作目录（即 zxx.options.GetPath() 的返回值）。
        """
        self.__source_file = File(filename, folder)
        return self

    def show_score(self, show: bool = True) -> Highlight:
        """
        设置是否显示比分牌。
        返回值是 self，即调用该函数的实例本身，因此可用于链式写法。
        """
        self.__show_score = show
        return self

    def set_score(self, home: int, away: int) -> Highlight:
        """
        更新比分。
        返回值是 self，即调用该函数的实例本身，因此可用于链式写法。

        参数说明：
            home：更新后的主队得分。
            away：更新后的客队得分。
        """
        self.__score = (home, away)
        return self

    def take(self, *args: list) -> Highlight:
        """
        从正在使用的视频中提取片段，并指定后期处理选项，然后添加至集锦的末尾。
        如果新追加视频画面尺寸不同，会自动将新视频转化成和已有集锦相同的尺寸。
        返回值是 self，即调用该函数的实例本身，因此可用于链式写法。
        
        这一方法的参数是 *args，即可以传入任意数量的参数。具体用法如下：

            1. 如果不传入任何参数，则将当前正在使用的视频不作任何剪辑或后期处理，直接添加至集锦的末尾。
            2. 如果传入参数，每个参数必须是 list。每个 list 传递从视频文件中截取的一个片段和对应的后期处理信息。
            3. 每个 list 内部元素数量必须是 1 至 4 个：
                (a) 如果是 1 个字符串元素，则代表将当前正在使用的视频不作任何剪辑，只添加字幕，然后添加至集锦的末尾。例如：
                    Highlight().use("素材.mp4").take(["这是一段解说文字！"])
                (b) 如果是 1 个字典元素，则代表将当前正在使用的视频不作任何剪辑，只添加特效，然后添加至集锦的末尾。例如：
                    Highlight().use("素材.mp4").take([{"speed": 0.5, "fadein": 0.2}])
                (c) 如果是 2 个字符串元素，必须是起始时刻和结束时刻，代表将当前正在使用的视频按照该时间范围进行剪辑，不添加任何字幕或特效，然后添加至集锦的末尾。开始 / 结束时刻如果是空字符串，默认是视频的开始 / 结束处。例如：
                    Highlight().use("素材.mp4").take(["00:06", "00:20"])
                    Highlight().use("素材.mp4").take(["", "00:20"])
                    Highlight().use("素材.mp4").take(["00:06", ""])
                (d) 如果是 2 个元素，也可以按照先字幕、后特效的顺序指定，例如：
                    Highlight().use("素材.mp4").take(["这是一段解说文字！", {"speed": 0.5, "fadein": 0.2}])
                (e) 如果是 3 个元素，前 2 个元素必须是起始时刻和结束时刻，第 3 个元素是字幕或特效。例如：
                    Highlight().use("素材.mp4").take(["00:06", "00:20", "这是一段解说文字！"])
                    Highlight().use("素材.mp4").take(["00:06", "", {"speed": 0.5, "fadein": 0.2}])
                (f) 如果是 4 个元素，必须按照起始时刻、结束时刻、字幕、特效的顺序排列。例如：
                    Highlight().use("素材.mp4").take(["00:06", "", "这是一段解说文字！", {"speed": 0.5, "fadein": 0.2}])
            4. 特效元素（dict 类型）会传递到 zxx.tools.effects() 方法作为参数。
                每种特效的键名、值类型和默认值分别为：
                    speed: float = 1, 
                    silence: bool = False, 
                    lum: float = 0, 
                    contrast: float = 0, 
                    fadein: float = 0, 
                    fadeout: float = 0,
                具体意义：
                    speed：调整视频的速度倍数，不建议放慢太多，可能导致时间轴错误。
                    silence：是否消音。
                    lum：亮度要增加或减少的值，大小没有限制，不过一般在 -127 至 127 之间，可自行调试。
                    contrast：对比度要调整的值，大小没有限制，不过一般在 -1 至 1之间，可自行调试。
                    fadein：淡入效果持续的秒数。
                    fadeout：淡出效果持续的秒数。
        """
        # 不传入任何参数的情况
        if len(args) == 0:
            args = [["", ""]]
        # 分析参数
        for info in args:
            if len(info) >= 5:
                raise Exception("*args 中某一项包含元素过多，必须是 1 至 4 个！")
            if len(info) == 1:    # 处理只传入 1 个元素的情况
                info = ["", "", info[0]]
            elif len(info) == 2:    # 处理传入 2 个元素为字幕和特效的情况
                if isinstance(info[1], dict):
                    info = ["", "", info[0], info[1]]
            begin = info[0]
            end = info[1]
            if begin == "":
                begin = "0"
            if end == "":
                end = str(self.source_file().contents().duration)
            clip = self.source_file().select(begin, end)
            if len(info) == 4:    # 添加特效和字幕，顺序是先加特效，再加字幕和比分牌
                effects = info[3]
                clip = add_effects(clip, **effects)
                caption = info[2]
                clip = add_caption(clip, caption)
            elif len(info) == 3:    # 只添加特效或字幕
                if isinstance(info[2], dict):
                    effects = info[2]
                    clip = add_effects(clip, **effects)
                else:
                    caption = info[2]
                    clip = add_caption(clip, caption)
            if self.__show_score:
                home_score, away_score = self.score()
                clip = add_scoreboard(clip, home_score, away_score)
            self.__add_video(clip)
        return self

    def export(self, filename: str, folder: str = None, mode: str = "hd", threads: int | None = None) -> Highlight:
        """
        将整个集锦导出成视频文件。
        返回值是 self，即调用该函数的实例本身，因此可用于链式写法。

        参数说明：
            filename：导出的视频文件名。
            folder：导出的视频文件所在的文件夹绝对路径。如果未指定，则默认为工作目录（即 zxx.options.GetPath() 的返回值）。
            mode：导出模式。可设置以下模式：
                mode = "preview"：用于导出快速预览，视频文件会很小，建议导出文件名选择 .mp4 后缀。
                mode = "hd"：高清画质，视频压缩效果好，建议导出文件名选择 .mp4 后缀。
                mode = "DJI Action 4"：匹配大疆 Action 4 画质，建议导出文件名选择 .mp4 后缀。
                mode = "lossless"，无损画质，文件非常大，不推荐使用。建议导出文件名选择 .avi 
            threads：导出时的线程数，默认为 None 即不使用多线程。
        """
        if folder == None:
            folder = GetPath()
        output_path = path.join(folder, filename)
        # 快速导出
        if mode == "preview":
            self.__contents.write_videofile(
                output_path,
                fps = 24,
                preset = "ultrafast",
                bitrate = "1000k",
                threads = threads,
            )
        # 高清画质
        ## 视频质量通过 bitrate 参数调节，B站推荐4k视频码率大于20000kbps
        elif mode == "hd":
            self.__contents.write_videofile(
                output_path,
                codec = "libx264",
                bitrate = "20000k",
                threads = threads,
            )
        # 匹配大疆 Action 4 画质
        elif mode == "DJI Action 4":
            self.__contents.write_videofile(
                output_path,
                fps = 60,
                codec="libx264",
                bitrate="100000k",
                threads = threads,
            )
        # 无损画质
        elif mode == "lossless":
            self.__contents.write_videofile(
                output_path,
                codec = "png",
                bitrate = "20000k",
                threads = threads,
            )
        print("视频已导出至 %s" % output_path)
        return self
    
    def change_match_info(self, home: str = "", away: str = "") -> Highlight:
        """
        中途修改比赛信息（主客队信息），用于制作多场比赛的集锦。
        """
        if home != "" and away != "":
            SetMatchInfo(home=home, away=away)
        elif home != "":
            SetMatchInfo(home=home)
        elif away != "":
            SetMatchInfo(away=away)
        return self



"""
zxx.options
全局设置
    存储全局设置的一些参数，以及修改这些参数的接口。
"""


# 以下全局设置相关的参数，及其默认值
# 想要查看或修改这些参数，请勿直接调用这些变量，而是应当使用本模块提供的相应函数。

PATH = "D:\\"

MATCH_INFO = {
    "home": "政信信",
    "away": "",
}

CAPTION_STYLE = {
    "font": "楷体",
    "fontsize": 100,    # 一行大约19个汉字
    "color": "white",
    "position": "bottom",
    "relative": False,
}

SCOREBOARD_STYLE = {
    "image": "scoreboard.png",
    "font_file": "华文新魏.ttf",
    "color": "black",
}


# 以下是修改设置的函数

def SetPath(path: str) -> None:
    """
    设置工作路径（绝对路径）。
    """
    global PATH
    PATH = path


def SetMatchInfo(**kwargs) -> None:
    """
    设置比赛信息。可以传入任意的参数，无效的参数会被忽略。

    可以识别的参数及其默认值：
        home="政信信",
        away="",

    参数说明：
        指定主队和客队的队名，会显示在比分牌上。
    """
    global MATCH_INFO
    MATCH_INFO.update(kwargs)


def SetCaptionStyle(**kwargs) -> None:
    """
    修改字幕设置。可以传入任意的参数，无效的参数会被忽略。

    可以传入的参数及其默认值：
        font="楷体",
        fontsize=100,
        color="white",
        position="bottom",
        relative=False,

    参数说明：
        font、fontsize 和 color 参数都是传入 moviepy.video.VideoClip 的 TextClip 类的初始化参数 font。
        font：直接键入字体名即可，但必须是在 zxx.options.AllFonts() 返回的字体列表中的字体名。
        fontsize：字号，可调试后调整数字大小。
        color：字体颜色。须符合 ImageMagick 的颜色写法，使用色彩名、十六进制、RGB、RGBA 等均可，
            可采用以下写法中的任意一种：
                color="lime"
                color="#00ff00"
                color="rgb(0, 255, 0)"
                color="rgba(0, 255, 0, 1.0)"
            详见：https://imagemagick.org/script/color.php
        position 和 relative：传入 TextClip.set_position() 函数，可以使用多种写法。
            set_position() 函数示例翻译如下：
                clip.set_position((45,150))    # 像素坐标 x=45, y=150
                clip.set_position(("center","top"))    # 水平居中，竖直对齐
                clip.set_position((0.4,0.7), relative=True)    # 相对位置，水平 40%，竖直 70%
                clip.set_position(lambda t: ('center', 50+t) )    # 水平居中，竖直向上移动
            对应地，使用本函数时，直接写成：
                position=(45,150)
                position="bottom"
                position=("center","top")
                position=(0.4,0.7), relative=True
                position=lambda t: ('center', 50+t)
    """
    global CAPTION_STYLE
    CAPTION_STYLE.update(kwargs)


def SetScoreBoardStyle(**kwargs) -> None:
    """
    修改比分牌设置。可以传入任意的参数，无效的参数会被忽略。

    可以传入的参数及其默认值：
        image="scoreboard.png",
        font_file="华文新魏.ttf",
        color="black",

    参数说明：
        image：比分牌图片的文件名，工作路径（即 zxx.options.GetPath() 返回值）的相对路径。最好是细长的横版图片。
        font_file：字体文件名，需要把字体文件复制到工作路径。
        color：字体颜色。须符合 Pillow 库的颜色写法，使用色彩名、十六进制、RGB、RGBA 等均可，
            可采用以下写法中的任意一种：
                color="pink"
                color="#800080"
                color=(255, 10, 10)
                color=(255, 10, 10, 100)
    """
    global SCOREBOARD_STYLE
    SCOREBOARD_STYLE.update(kwargs)


# 以下是读取设置的函数

def GetPath() -> str:
    """
    返回当前设置的工作路径。
    """
    return PATH


def GetMatchInfo(para: str = None) -> dict | str:
    """
    返回比赛信息。
    如果具体指定需要某个参数，则返回该参数的值；如果未指定，返回所有比赛信息。
    """
    if para == None:
        return MATCH_INFO
    
    else:
        return MATCH_INFO[para]


def GetCaptionStyle(para: str = None) -> dict | str:
    """
    返回字幕设置。
    如果具体指定需要某个参数，则返回该参数的值；如果未指定，返回所有字幕设置。
    """
    if para == None:
        return CAPTION_STYLE
    
    else:
        return CAPTION_STYLE[para]
 
    
def GetScoreBoardStyle(para: str = None) -> dict | str:
    """
    返回比分牌设置。
    如果具体指定需要某个参数，则返回该参数的值；如果未指定，返回所有比分牌设置。
    """
    if para == None:
        return SCOREBOARD_STYLE
    
    else:
        return SCOREBOARD_STYLE[para]


def AllFonts(print_list: bool = True) -> list:
    """
    返回所有可用的字体列表。

    参数说明：
        print_list：是否打印字体列表，默认打印。
    """
    from moviepy.editor import TextClip

    font_list = TextClip.list("font")
    if print_list:
        print(font_list)
    
    return font_list
